import Builder from "../Builder";
import JSONConvertible from "./JSONConvertible";
/**
 * Defines an object that can be automatically built from JSON data.
 *
 * Use this as the base class for your resources (ie. Users, Articles, Posts, ect.)
 */
export default abstract class BuildableResource<Type extends BuildableResource<Type> = any> implements JSONConvertible {
    /**
     * The specified {@link Builder} for this class.
     */
    private builder?;
    /**
     * The base constructor with no arguments. Your class NEEDS to have it.
     */
    constructor();
    /**
     * Public getter for this class' current builder.
     */
    get build(): Builder<this>;
    /**
     * A public setter for this class' current builder.
     */
    set newBuilder(builder: Builder<this>);
    fromJSON(source: any, strict?: boolean): this;
    toJSON(): any;
}
//# sourceMappingURL=BuildableResource.d.ts.map