"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Builder_1 = __importDefault(require("../Builder"));
var JSONConvertible_1 = __importDefault(require("./JSONConvertible"));
var functions_1 = require("../helpers/functions");
/**
 * Defines an object that can be automatically built from JSON data.
 *
 * Use this as the base class for your resources (ie. Users, Articles, Posts, ect.)
 */
var BuildableResource = /** @class */ (function () {
    /**
     * The base constructor with no arguments. Your class NEEDS to have it.
     */
    function BuildableResource() {
    }
    Object.defineProperty(BuildableResource.prototype, "build", {
        /**
         * Public getter for this class' current builder.
         */
        get: function () {
            var c = this.constructor.prototype.constructor;
            if (!Object.getOwnPropertyDescriptor(this, 'builder') && this.builder) {
                this.builder = (0, functions_1.deepCopy)(this.builder);
                if (!this.builder.isUsingClass(c)) {
                    this.builder.updateBaseObject(c);
                }
            }
            else if (!this.builder) {
                this.builder = new Builder_1.default(c);
            }
            return this.builder;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BuildableResource.prototype, "newBuilder", {
        /**
         * A public setter for this class' current builder.
         */
        set: function (builder) {
            this.builder = builder;
        },
        enumerable: false,
        configurable: true
    });
    BuildableResource.prototype.fromJSON = function (source, strict) {
        if (strict === void 0) { strict = false; }
        return this.build.fromJSON(source, strict);
    };
    BuildableResource.prototype.toJSON = function () {
        return this.build.toJSON(this);
    };
    return BuildableResource;
}());
exports.default = BuildableResource;
//# sourceMappingURL=BuildableResource.js.map