"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListOf = exports.Ignore = exports.Transform = exports.Alias = exports.Resource = void 0;
var BuildableResource_1 = __importDefault(require("./contracts/BuildableResource"));
var ResourceFactory_1 = __importDefault(require("./contracts/ResourceFactory"));
var ResourceDecorator = /** @class */ (function () {
    function ResourceDecorator() {
    }
    /**
     * Define a class as a resource and check for its validity.
     */
    ResourceDecorator.prototype.Resource = function (constructor) {
        var target = constructor.prototype;
        if (!(target instanceof BuildableResource_1.default)) {
            throw new Error("Target class has to extend BuildableResource in order to use this decorator.");
        }
        if (!target.constructor) {
            throw new Error("Target class must have a constructor with no arguments.");
        }
    };
    /**
     * Adds an alias.
     * @param alias The alias to add.
     */
    ResourceDecorator.prototype.Alias = function (alias) {
        return function (target, name) {
            var _a;
            (_a = target.build) === null || _a === void 0 ? void 0 : _a.alias(alias, name.toString());
        };
    };
    /**
     * Adds a transformer.
     * @param transformerIn The transformer for incoming properties.
     * @param transformerOut The transformer for outgoing properties.
     */
    ResourceDecorator.prototype.Transform = function (transformerIn, transformerOut) {
        return function (target, name) {
            var _a;
            (_a = target.build) === null || _a === void 0 ? void 0 : _a.transform(name, transformerIn, transformerOut);
        };
    };
    /**
     * Adds an ignore directive.
     */
    ResourceDecorator.prototype.Ignore = function (target, name) {
        var _a;
        (_a = target.build) === null || _a === void 0 ? void 0 : _a.ignore(name.toString());
    };
    //NOTE we can use the Resource decorator for the same purpose
    /**
     * Define a list of {@link BuildableResource}(s).
     * @param resource The list items' class.
     */
    ResourceDecorator.prototype.ListOf = function (resource) {
        return function (target, name) {
            var _a;
            (_a = target.build) === null || _a === void 0 ? void 0 : _a.listType(name, new resource());
        };
    };
    return ResourceDecorator;
}());
var resourceDecorator = new ResourceDecorator();
exports.Resource = resourceDecorator.Resource;
exports.Alias = resourceDecorator.Alias;
exports.Transform = resourceDecorator.Transform;
exports.Ignore = resourceDecorator.Ignore;
exports.ListOf = resourceDecorator.ListOf;
//# sourceMappingURL=decorators.js.map